@@ -1,6 +1,7 @@
 # wrapper to apply genomic prediction models to an object of class gpData
-# models used: regress and BLR
+# models used: regress/asreml and BLR
 
+# date: 2011 - 01 - 16
 # author: Valentin Wimmer
 # date: 2011 - 05 - 03
 # changes: Hans-Juergen Auinger
@@ -8,17 +9,25 @@
 # changes: return argument by Valentin Wimmer
 # date: 2011 - 11 - 30
 # changes: Hans-Juergen Auinger
-# date: 2011 - 01 - 16
+# date: 2012 - 10 - 26
+# changes: including R-asreml for BLUP by Hans-Juergen Auinger
 
 gpMod <- function(gpData,model=c("BLUP","BL","BRR"),kin=NULL,predict=FALSE,trait=1,repl=NULL,markerEffects=FALSE,fixed=NULL,random=NULL,...){
-  ASReml <- FALSE
+
+  ASReml <- "package:asreml" %in% search()
   ans <- list()
   model <- match.arg(model)
   m <- NULL
   prediction <- NULL
   if(length(trait) > 1) cat("\n\tNOTE:   The return object will be a list of gpMod-objects!\n")
-  if(is.null(fixed)) fixed <- " ~ 1"
-  if(is.null(random)) random <- "~ " else random <- paste(paste(random, collapse=" "), "+ ")
+  if(is.null(fixed)) fixed <- ~1
+  if(is.null(random)) {
+    random <- " ~ "
+    randomFormula <- ~1
+  } else {
+    randomFormula <- random
+    random <- paste(paste(random, collapse=" "), "+ ")
+  }
   if(model == "BLUP"){
     if (is.null(kin)){
       if(!gpData$info$codeGeno) stop("Missing object 'kin', or use function codeGeno first!")
@@ -31,7 +40,7 @@ gpMod <- function(gpData,model=c("BLUP","BL","BRR"),kin=NULL,predict=FALSE,trait
   for(i in trait){
     df.trait <- gpData2data.frame(gpData, i, onlyPheno=TRUE, repl=repl)
     # take data from gpData object
-    vec.bool <- colnames(df.trait) == "ID" | colnames(df.trait) %in% unlist(strsplit(paste(fixed), " ")) | colnames(df.trait) %in% unlist(strsplit(paste(random), " "))
+    vec.bool <- colnames(df.trait) == "ID" | (colnames(df.trait) %in% dimnames(attr(terms(fixed), "factors"))[[1]]) | colnames(df.trait) %in% dimnames(attr(terms(randomFormula), "factors"))[[1]]
     cnt <- dim(gpData$pheno)[2]
     if(!is.null(gpData$phenoCovars)) cnt <- cnt + dim(gpData$phenoCovars)[2]
     if(i %in% 1:cnt) {
@@ -58,36 +67,25 @@ gpMod <- function(gpData,model=c("BLUP","BL","BRR"),kin=NULL,predict=FALSE,trait
         genVal <- genVal[!duplicated(names(genVal))]
         names(genVal) <-  unlist(strsplit(names(genVal), "kinTS."))[(1:length(genVal))*2]
       } else {
-        system("mkdir ASReml")
-        oldwd <- getwd()
-        setwd(paste(oldwd, "ASReml", sep="/"))
         kinTS <- kinNew[dimnames(gpData$pheno)[[1]], dimnames(gpData$pheno)[[1]]]
-        write.relationshipMatrix(kinTS+diag(x=10**(-7), nrow(kinTS)),file="Kins.giv",type="inv",sorting="ASReml",digits=10)
-        write.table(df.trait, file="Pheno.txt", col.names=TRUE, row.names=FALSE, quote=FALSE, sep="\t")
-        cat("Model \n ID        !A \n TRAIT     !D* \n", file = "Model.as")
-        if(ncol(df.trait) > 2) for(i in 3:ncol(df.trait)){
-          cat(" ", colnames(df.trait)[i], sep= "", file = "Model.as", append=TRUE)
-          if(gpData$info$attrPhenoCovars[colnames(df.trait)[i]] == "numeric") cat("   !D*\n", file="Model.as",append=TRUE) else cat("   !A*\n", file="Model.as",append=TRUE)
+        covM.I <- try(solve(kinTS),TRUE)
+        # adding constant to diagonal, if covM is singular
+        if(class(covM.I)=="try-error"){
+          warning("Covariance matrix is computationally singular: constant 1e-6 is added to the diagonal elements of the covariance matrix")
+          covM.I <- solve(kinTS + diag(1e-6,ncol(kinTS)))
         }
-        if(!is.null(fixed)){
-          fixedC <- as.character(fixed)[2]
-          fixedC <- paste(unlist(strsplit(fixedC, "+", fixed=TRUE)), collapse=" ")
-        } else fixedC <- NULL
-        if(!is.null(random)){
-          randomC <- as.character(random)[2]
-          randomC <- paste(unlist(strsplit(randomC, "+", fixed=TRUE)), collapse=" ")
-        } else randomC <- NULL
-        cat(paste("Kins.giv \nPheno.txt !skip 1 !AISING !maxit 11\n!MVINCLUDE \n \nTRAIT  ~ mu ", fixedC,
-            " !r ", random, " giv(ID,1)",sep=""),file="Model.as", append=TRUE)
-        cat("",file="Model.pin")
-        system("asreml -ns10000 Model.as",TRUE)
-        system("asreml -p Model.pin") # for variance components in an extra file
-        us <- matrix(scan("Model.sln",what='character'),ncol=4,byrow=TRUE)
-        # solve MME
-        genVal <-  as.numeric(us[us[, 1] == "giv(ID,1)",3])
-        names(genVal) <- us[us[, 1] == "giv(ID,1)",2]
-        res <- scan("Model.asr", what="character", sep="\n")
-        setwd(oldwd)
+        # print warning in case of numerical problems
+        if(any(covM.I>1e8)) warning("Large >1e8 entries in the inverse covariance matrix")
+        kinGinv <- write.relationshipMatrix(covM.I,file=NULL,type="none",sorting="ASReml",digits=10)
+        attr(kinGinv, "rowNames") <- rownames(kinTS)
+
+        if("ginverse" %in% names(list(...)))
+          asrObj <- asreml(as.formula(paste(yName, paste(fixed, collapse=""))), random=as.formula(paste(random, "giv(ID, var=TRUE)")), ginverse=c(list(ID=kinGinv), ginverse), data=df.trait)
+        else
+          asrObj <- asreml(as.formula(paste(yName, paste(fixed, collapse=""))), random=as.formula(paste(random, "giv(ID, var=TRUE)")), ginverse=list(ID=kinGinv), data=df.trait)
+
+        genVal <-  asrObj$coefficients$random[substr(names(asrObj$coefficients$random), 1, 8) == "giv(ID)_"]
+        names(genVal) <- substr(names(genVal), 9, nchar(names(genVal)))
       }
       # genVal <- NULL
       if(markerEffects){
@@ -107,12 +105,25 @@ gpMod <- function(gpData,model=c("BLUP","BL","BRR"),kin=NULL,predict=FALSE,trait
     }
 
     if(model=="BL"){
+      if(random != "~ ") stop("In the method BL random terms are not supported!")
       if(dim(gpData$pheno)[3] > 1) stop("This method is not developed for a one-stage analysis yet. \nA phenotypic analysis have to be done fist.")
       X <- gpData$geno[rownames(gpData$geno) %in% df.trait$ID,]
       y <- df.trait[df.trait$ID %in% rownames(gpData$geno), yName]
-      capture.output(res <- BLR(y=y,XL=X,...),file="BLRout.txt")
-      if(!is.null(kin)) capture.output(res <- BLR(y=y,XL=X,GF=list(ID=df.trait$ID,A=kinTS),...),file="BLRout.txt")
-      else kin <- NULL
+      if(fixed != " ~ 1"){
+        XF <- as.formula(paste(yName, paste(fixed, collapse=" ")))
+        XF <- model.matrix(XF,data=df.trait[df.trait$ID %in% rownames(gpData$geno),],na.action=na.pass)
+      } else XF <- NULL
+      if(is.null(kin)){
+        if(is.null(XF))
+          capture.output(res <- BLR(y=y,XL=X,...),file="BLRout.txt")
+        else
+          capture.output(res <- BLR(y=y,XL=X,XF=XF,...),file="BLRout.txt")
+      } else {
+        if(is.null(XF))
+          capture.output(res <- BLR(y=y,XL=X,GF=list(ID=df.trait$ID,A=kinTS),...),file="BLRout.txt")
+        else
+          capture.output(res <- BLR(y=y,XL=X,GF=list(ID=df.trait$ID,A=kinTS),XF=XF,...),file="BLRout.txt")
+      }
       genVal <- res$yHat
       names(genVal) <- rownames(X)
       m <- res$bL
@@ -122,12 +133,25 @@ gpMod <- function(gpData,model=c("BLUP","BL","BRR"),kin=NULL,predict=FALSE,trait
       }
     }
     if(model=="BRR"){
+      if(random != "~ ") stop("In the method BL random terms are not supported!")
       if(dim(gpData$pheno)[3] > 1) stop("This method is not developed for a one-stage analysis yet. \nA phenotypic analysis has to be done fist.")
       X <-  gpData$geno[rownames(gpData$geno) %in% df.trait$ID,]
       y <- df.trait[df.trait$ID %in% rownames(gpData$geno), yName]
-      capture.output(res <- BLR(y=y,XR=X,...),file="BLRout.txt")
-      if(!is.null(kin))  capture.output(res <- BLR(y=y,XR=X,GF=list(ID=df.trait$ID,A=kinTS),...),file="BLRout.txt")
-      else kin <- NULL
+      if(fixed != " ~ 1"){
+        XF <- as.formula(paste(yName, paste(fixed, collapse=" ")))
+        XF <- model.matrix(XF,data=df.trait[df.trait$ID %in% rownames(gpData$geno),],na.action=na.pass)
+      } else XF <- NULL
+      if(is.null(kin)){
+        if(is.null(XF))
+          capture.output(res <- BLR(y=y,XR=X,...),file="BLRout.txt")
+        else
+          capture.output(res <- BLR(y=y,XR=X,XF=XF,...),file="BLRout.txt")
+      } else {
+        if(is.null(XF))
+          capture.output(res <- BLR(y=y,XR=X,GF=list(ID=df.trait$ID,A=kinTS),...),file="BLRout.txt")
+        else
+          capture.output(res <- BLR(y=y,XR=X,GF=list(ID=df.trait$ID,A=kinTS),XF=XF,...),file="BLRout.txt")
+      }
       genVal <- res$yHat
       names(genVal) <- rownames(X)
       m <- res$bR
